from typing import List

from yelp_fetcher.reviews import Review
from yelp_lambda.common_types import JobType, LambdaMessage


class WorkerRequest(LambdaMessage):
    def __init__(self, job_type):
        super().__init__(job_type)


class ReviewsWorkerRequest(WorkerRequest):
    def __init__(self, user_id, url):
        super().__init__(JobType.REVIEWS.value)
        self.user_id = user_id
        self.url = url


class StatusesWorkerRequest(WorkerRequest):
    def __init__(self, user_id, biz_id, review_id):
        super().__init__(JobType.STATUSES.value)
        self.user_id = user_id
        self.biz_id = biz_id
        self.review_id = review_id


class WorkerResponse(LambdaMessage):
    def __init__(self, job_type):
        super().__init__(job_type)


class ReviewsWorkerResult(WorkerResponse):
    def __init__(self, user_id, reviews: List[Review]):
        super().__init__(JobType.REVIEWS.value)
        self.user_id = user_id
        self.reviews = reviews


class StatusesWorkerResult(WorkerResponse):
    def __init__(self, user_id, biz_id, review_id, status):
        super().__init__(JobType.STATUSES.value)
        self.user_id = user_id
        self.biz_id = biz_id
        self.review_id = review_id
        self.status = status



from enum import Enum
from typing import TypedDict

from yelp_fetcher._util import FetchError, fetch_html
from yelp_fetcher.scraper.biz import get_biz_review_url, review_id_on_page


class Status(Enum):
    ALIVE = "ALIVE"
    DEAD = "DEAD"
    UNKNOWN = "UNKNOWN"


class ReviewStatus(TypedDict):
    user_id: str
    biz_id: str
    review_id: str
    status: str


def fetch_status(user_id, biz_id, review_id):
    url = get_biz_review_url(biz_id, review_id)
    try:
        html = fetch_html(url)
        status = Status.ALIVE if review_id_on_page(html, review_id) else Status.DEAD
    except FetchError:
        status = Status.UNKNOWN

    return ReviewStatus(user_id=user_id, biz_id=biz_id, review_id=review_id, status=status.value)



from yelp_lambda.common_types import (
    BIZ_ID,
    REVIEW_ID,
    URL,
    USER_ID,
    JobType,
    LambdaMessage,
)
from yelp_lambda.worker_types import ReviewsWorkerRequest, StatusesWorkerRequest


class DispatcherRequest(LambdaMessage):
    def __init__(self, job_type, args_list):
        super().__init__(job_type)
        self.args_list = args_list

    def get_worker_requests(self):
        if self.job_type == JobType.REVIEWS.value:
            worker_request_type = ReviewsWorkerRequest
        elif self.job_type == JobType.STATUSES.value:
            worker_request_type = StatusesWorkerRequest
        else:
            raise ValueError(f"Unsupported job_type: {self.job_type}")

        return [worker_request_type(**args) for args in self.args_list]


class ReviewsDispatcherRequest(DispatcherRequest):
    def __init__(self, user_id, urls):
        self.user_id = user_id
        self.urls = urls
        args_list = [{USER_ID: user_id, URL: url} for url in urls]
        super().__init__(JobType.REVIEWS.value, args_list)


class StatusesDispatcherRequest(DispatcherRequest):
    def __init__(self, user_id, biz_ids_to_review_ids):
        self.user_id = user_id
        self.biz_ids_to_review_ids = biz_ids_to_review_ids
        args_list = [
            {USER_ID: user_id, BIZ_ID: biz_id, REVIEW_ID: review_id,}
            for biz_id, review_id in biz_ids_to_review_ids.items()
        ]
        super().__init__(JobType.STATUSES.value, args_list)



from enum import Enum

JOB_TYPE = "job_type"

URL = "url"

USER_ID = "user_id"
BIZ_ID = "biz_id"
REVIEW_ID = "review_id"


class JobType(Enum):
    REVIEWS = "reviews"
    STATUSES = "statuses"


class LambdaMessage:
    def __init__(self, job_type):
        assert job_type
        self.job_type = job_type
